.section .aaa.5, "a"
.align 2
.quad 0x55

.section .aaa.1, "a"
.align 32
.quad 0x11

.section .aaa.3, "a"
.align 8
.quad 0x33

.section .aaa.2, "a"
.align 16
.quad 0x22

.section .aaa.4, "a"
.align 4
.quad 0x44
